@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2002,2004-2006,2010 Rami Lehti, Pablo Virolainen,
+ * Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -85,6 +86,10 @@ typedef struct xattrs_type
 #endif
 #endif
 
+#ifdef WITH_E2FSATTRS
+#include <e2p/e2p.h>
+#endif
+
 #ifdef WITH_MHASH
 #include <mhash.h>
 #endif
@@ -151,6 +156,7 @@ typedef enum {
    db_whirlpool,		/* "whirlpool",  */
    db_selinux, 			/* "selinux",  */
    db_xattrs, 			/* "xattrs",  */
+   db_e2fsattrs,        /* "e2fsattrs"     */
    db_unknown } DB_FIELD; 	/* "unknown"  */
 
 /* db_unknown must be last because it is used to determine size of
@@ -203,6 +209,8 @@ typedef enum {
 #define DB_SELINUX    (1LLU<<32) /* "selinux", */
 #define DB_XATTRS     (1LLU<<33) /* "xattrs",  */
 #define DB_WHIRLPOOL  (1LLU<<34) /* "whirlpool",  */
+#define DB_FTYPE      (1LLU<<35) /* "file type",  */
+#define DB_E2FSATTRS  (1LLU<<36) /* "ext2 file system attributes"  */
 
 #define DB_HASHES    (DB_MD5|DB_SHA1|DB_RMD160|DB_TIGER|DB_CRC32|DB_HAVAL| \
 		      DB_GOST|DB_CRC32B|DB_SHA256|DB_SHA512|DB_WHIRLPOOL)
@@ -339,6 +347,8 @@ typedef struct db_config {
 #endif
   int warn_dead_symlinks;
 
+  int grouped;
+
   int summarize_changes;
 
   struct seltree* tree;
@@ -383,6 +393,8 @@ typedef struct db_line {
  
   xattrs_type* xattrs;
 
+  unsigned long e2fsattrs;
+
   /* Attributes .... */
   DB_ATTR_TYPE attr;
   
