@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2002,2006,2010 Rami Lehti, Pablo Virolainen,
+ * Richard van den Berg, Hannes von Haugwitz
  * 
  * $Header$
  * This program is free software; you can redistribute it and/or
@@ -33,4 +34,8 @@ list* do_md(list* file_lst,db_config* conf);
 
 void acl2line(db_line* line);
 
+#ifdef WITH_E2FSATTRS
+void e2fsattrs2line(db_line* line);
+#endif
+
 #endif /* _DO_MD_H_INCLUDED */
