@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
+ * Copyright (C) 1999-2002,2006,2010 Rami Lehti,Pablo Virolainen,
+ * Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -33,6 +34,8 @@ typedef struct rx_rule {
   long  conf_lineno; /* line no. of rule definition*/
 } rx_rule;
 
+int compare_node_by_path(const void *n1, const void *n2);
+
 /*
  * gen_list()
  * Generates the file list
