@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2006,2010 Rami Lehti, Pablo Virolainen, Mike
+ * Markley, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -325,6 +326,8 @@ void setdefaults_before_config()
 
   conf->warn_dead_symlinks=0;
 
+  conf->grouped=1;
+
   conf->summarize_changes=0;
 
   conf->selrxlst=NULL;
@@ -375,6 +378,10 @@ void setdefaults_before_config()
 #ifdef HAVE_MHASH_WHIRLPOOL
   do_groupdef("whirlpool",DB_WHIRLPOOL);
 #endif
+#endif
+  do_groupdef("ftype",DB_FTYPE);
+#ifdef WITH_E2FSATTRS
+  do_groupdef("e2fsattrs",DB_E2FSATTRS);
 #endif
 
   p=0LLU;
@@ -390,7 +397,10 @@ void setdefaults_before_config()
 #ifdef WITH_XATTR
   p|=DB_XATTRS;
 #endif
-  do_groupdef("R",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZE|
+#ifdef WITH_E2FSATTRS
+  p|=DB_E2FSATTRS;
+#endif
+  do_groupdef("R",DB_PERM|DB_FTYPE|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZE|
                   DB_LINKNAME|DB_MTIME|DB_CTIME|p);
 
   p=0LLU;
@@ -403,7 +413,10 @@ void setdefaults_before_config()
 #ifdef WITH_XATTR
   p|=DB_XATTRS;
 #endif
-  do_groupdef("L",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_LINKNAME|p);
+#ifdef WITH_E2FSATTRS
+  p|=DB_E2FSATTRS;
+#endif
+  do_groupdef("L",DB_PERM|DB_FTYPE|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_LINKNAME|p);
 
   p=0LLU;
 #ifdef WITH_ACL
@@ -415,7 +428,10 @@ void setdefaults_before_config()
 #ifdef WITH_XATTR
   p|=DB_XATTRS;
 #endif
-  do_groupdef(">",DB_PERM|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZEG|
+#ifdef WITH_E2FSATTRS
+  p|=DB_E2FSATTRS;
+#endif
+  do_groupdef(">",DB_PERM|DB_FTYPE|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZEG|
 		  DB_LINKNAME|p);
   do_groupdef("E",0);
 
@@ -579,12 +595,17 @@ int main(int argc,char**argv)
     totalchanges=report_tree(conf->tree,0,status);
     if(totalchanges>0){
       exitcode=(status[2]!=0)*1+(status[3]!=0)*2+(status[4]!=0)*4;
+      if (conf->grouped) {
       status[0]=1;
       report_tree(conf->tree,1,status);
       status[0]=1;
       report_tree(conf->tree,2,status);
       status[0]=1;
       report_tree(conf->tree,3,status);
+      } else {
+          status[0]=1;
+          report_tree(conf->tree,5,status);
+      }
       /* Then detailed list of changes */
       status[0]=1;
       report_tree(conf->tree,4,status);
