@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2006,2010 Rami Lehti, Pablo Virolainen, Richard
+ * van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -428,9 +429,12 @@ void update_db_out_order(DB_ATTR_TYPE attr)
   if (check_dboo(db_attr)==RETOK) {
     conf->db_out_order[conf->db_out_size++]=db_attr;
   }
-  if((attr&DB_PERM) && (check_dboo(db_perm)!=RETFAIL)){
+  if(check_dboo(db_perm)==RETOK){
     conf->db_out_order[conf->db_out_size++]=db_perm;
   }
+  if(check_dboo(db_inode)==RETOK){
+    conf->db_out_order[conf->db_out_size++]=db_inode;
+  }
   if((attr&DB_BCOUNT) && (check_dboo(db_bcount)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_bcount;
   }
@@ -455,9 +459,6 @@ void update_db_out_order(DB_ATTR_TYPE attr)
   if((attr&DB_CTIME) && (check_dboo(db_ctime)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_ctime;
   }
-  if((attr&DB_INODE||attr&DB_CHECKINODE) && (check_dboo(db_inode)!=RETFAIL)){
-    conf->db_out_order[conf->db_out_size++]=db_inode;
-  }
   if((attr&DB_LNKCOUNT) && (check_dboo(db_lnkcount)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_lnkcount;
   }
@@ -511,6 +512,9 @@ void update_db_out_order(DB_ATTR_TYPE attr)
   if((attr&DB_SELINUX) && (check_dboo(db_selinux)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_selinux;
   }
+  if((attr&DB_E2FSATTRS) && (check_dboo(db_e2fsattrs)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_e2fsattrs;
+  }
   if((attr&DB_CHECKMASK) && (check_dboo(db_checkmask)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_checkmask;
   }
