@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2006 Rami Lehti, Pablo Virolainen, Richard van den Berg
+ * Copyright (C) 1999-2007,2010 Rami Lehti, Pablo Virolainen, Richard van
+ * den Berg, Mike Markley, Hannes von Haugwitz
  * $Id$
  *
  * This program is free software; you can redistribute it and/or
@@ -57,12 +58,23 @@ const int part_len = 33; /* usable length of line[] for most purposes */
 const int long_part_len = 129; /* length of line[] for link names and selinux contexts */
 char      oline[129];
 char      nline[129];
-const char* entry_format=        "  %-9s: %-33s, %s\n";
-const char* entry_format_justnew="  %-9s: %-33c  %s\n";
+const char* entry_format=        " %-9s: %-33s, %s\n";
+const char* entry_format_justnew=" %-9s: %-33c  %s\n";
+#ifdef WITH_E2FSATTRS
+    /* flag->character mappings defined in lib/e2p/pf.c (part of e2fsprogs-1.41.12 sources) */
+    unsigned long flag_bits[] = { EXT2_SECRM_FL, EXT2_UNRM_FL, EXT2_SYNC_FL, EXT2_DIRSYNC_FL, EXT2_IMMUTABLE_FL,
+        EXT2_APPEND_FL, EXT2_NODUMP_FL, EXT2_NOATIME_FL, EXT2_COMPR_FL, EXT2_COMPRBLK_FL,
+        EXT2_DIRTY_FL, EXT2_NOCOMPR_FL, EXT2_ECOMPR_FL, EXT3_JOURNAL_DATA_FL, EXT2_INDEX_FL,
+        EXT2_NOTAIL_FL, EXT2_TOPDIR_FL, EXT4_EXTENTS_FL, EXT4_HUGE_FILE_FL};
+    char flag_char[] = "suSDiadAcBZXEjItTeh";
+#endif
+/* The initial length of summary string, the final length depends on
+ * compile options */
+int summary_len = 13;
 /*************/
 
-static int get_ignorelist() {
-  int ignorelist;
+static DB_ATTR_TYPE get_ignorelist() {
+  DB_ATTR_TYPE ignorelist;
   ignorelist=get_groupval("ignore_list");
 
   if (ignorelist==-1) {
@@ -72,8 +84,8 @@ static int get_ignorelist() {
   return ignorelist;
 }
 
-static int get_report_attributes() {
-  int forced_attrs;
+static DB_ATTR_TYPE get_report_attributes() {
+  DB_ATTR_TYPE forced_attrs;
   
   forced_attrs=get_groupval("report_attributes");
   if (forced_attrs==-1) {
@@ -266,6 +278,24 @@ static int compare_str(const char *s1, const char *s2)
   return RETOK;
 }
 
+char get_file_type_char(mode_t mode) {
+    if (S_ISREG(mode)) return 'f';
+    else if(S_ISDIR(mode)) return 'd';
+#ifdef S_ISFIFO
+    else if (S_ISFIFO(mode)) return 'F';
+#endif
+    else if (S_ISLNK(mode)) return 'L';
+    else if (S_ISBLK(mode)) return 'B';
+    else if (S_ISCHR(mode)) return 'D';
+#ifdef S_ISSOCK
+    else if (S_ISSOCK(mode)) return 's';
+#endif
+#ifdef S_ISDOOR
+    else if (S_ISDOOR(mode)) return '|';
+#endif
+    else return '?';
+}
+
 
 /*
   We assume
@@ -293,25 +323,30 @@ DB_ATTR_TYPE compare_dbline(db_line* l1,db_line* l2,DB_ATTR_TYPE ignorelist)
     } \
   }
   
-  
   DB_ATTR_TYPE ret=0;
   
+  if (!(DB_FTYPE&ignorelist)) {
+      if ((DB_FTYPE&l1->attr && DB_FTYPE&l2->attr) && get_file_type_char(l1->perm)!=get_file_type_char(l2->perm)) {
+	    ret|=DB_FTYPE;
+      }
+  }
+
   if (!(DB_LINKNAME&ignorelist)) {
     if(compare_str(l1->linkname, l2->linkname)){
 	ret|=DB_LINKNAME;
     }
   }
-    
+
   if (!(DB_SIZEG&ignorelist)) {
-    if ( (DB_SIZEG&l2->attr) && !(DB_SIZE&l2->attr) ){
-      if(l1->size>l2->size){
-	ret|=DB_SIZEG;
+      if ((DB_SIZEG&l1->attr && (DB_SIZEG&l2->attr))  && l1->size>l2->size){
+          ret|=DB_SIZEG;
       }
-    } else {
-      if(l1->size!=l2->size){
-	ret|=DB_SIZEG;
+  }
+
+  if (!(DB_SIZE&ignorelist)) {
+      if ((DB_SIZE&l1->attr && DB_SIZE&l2->attr) && l1->size!=l2->size){
+          ret|=DB_SIZE;
       }
-    }
   }
   
   easy_compare(DB_BCOUNT,bcount);
@@ -330,8 +365,11 @@ DB_ATTR_TYPE compare_dbline(db_line* l1,db_line* l2,DB_ATTR_TYPE ignorelist)
   easy_compare(DB_MTIME,mtime);
   easy_compare(DB_CTIME,ctime);
 
-
-  easy_compare(DB_INODE,inode);
+  if (!(DB_INODE&ignorelist)) {
+      if ((DB_INODE&l1->attr && DB_INODE&l2->attr) && (l1->inode!=l2->inode)){
+          ret|=DB_INODE;
+      }
+  }
   easy_compare(DB_LNKCOUNT,nlink);
 
   easy_md_compare(DB_MD5,md5,HASH_MD5_LEN);
@@ -366,6 +404,9 @@ DB_ATTR_TYPE compare_dbline(db_line* l1,db_line* l2,DB_ATTR_TYPE ignorelist)
       ret|=DB_XATTRS;
     }
   }
+#ifdef WITH_E2FSATTRS
+  easy_compare(DB_E2FSATTRS,e2fsattrs);
+#endif
   if (!(DB_SELINUX&ignorelist)) {
     if(compare_str(l1->cntx,l2->cntx)) {
       ret|=DB_SELINUX;
@@ -374,28 +415,32 @@ DB_ATTR_TYPE compare_dbline(db_line* l1,db_line* l2,DB_ATTR_TYPE ignorelist)
   return ret;
 }
 
-void print_str_changes(char*old,char*new,const char *name)
+void print_str_changes(char*old,char*new,const char *name, DB_ATTR_TYPE force)
 {
-  int ok = 0;
-
+  int mode = 0;
   if(old==NULL){
     if(new!=NULL){
        snprintf(oline,long_part_len,"<NULL>");
        snprintf(nline,long_part_len,"%s",new);
-       ok = 1;
+       mode = 1;
     }
   } else if(new==NULL){
        snprintf(oline,long_part_len,"%s",old);
        snprintf(nline,long_part_len,"<NULL>");
-       ok = 1;
+       mode = 1;
    } else if(strcmp(old,new)!=0){
         snprintf(oline,long_part_len,"%s",old);
         snprintf(nline,long_part_len,"%s",new);
-        ok = 1;
+        mode = 1;
+  } else if (force) {
+      snprintf(nline,long_part_len,"%s",new);
+      mode = 2;
   }
-   if(ok)
+   if(mode == 1) {
      error(2,(char*)entry_format,name,oline,nline);
-
+   } else if (mode == 2) {
+    error(2,(char*)entry_format_justnew,name,' ',nline);
+   }
    return;
 }
 
@@ -407,11 +452,11 @@ void print_single_acl(acl_type* acl)
   } else {
 #ifdef WITH_POSIX_ACL
     if (!acl->acl_a)
-      error(2,"A:<NONE>\n           ");
+      error(2,"A: <NONE>\n                  ");
     else
-      error(2,"A:\n----\n%s----\n           ",acl->acl_a);
+      error(2,"A:\n----\n%s----\n                  ",acl->acl_a);
     if (!acl->acl_d)
-      error(2,"D:<NONE>\n");
+      error(2,"D: <NONE>\n");
     else
       error(2,"D:\n----\n%s----\n",acl->acl_d);
 #endif
@@ -427,13 +472,16 @@ void print_single_acl(acl_type* acl)
   }
 }
 
-void print_acl_changes(acl_type* old,acl_type* new) {
+void print_acl_changes(acl_type* old,acl_type* new, DB_ATTR_TYPE force) {
   
   if (compare_acl(old,new)==RETFAIL) {
-    error(2,"ACL: old = ");
+    error(2," ACL      : old = ");
     print_single_acl(old);
-    error(2,"     new = ");
+    error(2,"            new = ");
     print_single_acl(new);
+  } else if (old!=NULL && new!=NULL && force) {
+      error(2," ACL      :       ");
+      print_single_acl(new);
   }
 }
 #endif
@@ -477,13 +525,13 @@ void print_single_xattrs(xattrs_type* xattrs)
       
       if ((len ==  xattrs->ents[num - 1].vsz) ||
           ((len == (xattrs->ents[num - 1].vsz - 1)) && !val[len]))
-        error(2,"  [%.*zd] %s = %s\n", width, num,
+        error(2,"             [%.*zd] %s = %s\n", width, num,
               xattrs->ents[num - 1].key, val);
       else        
       {
         val = encode_base64(xattrs->ents[num - 1].val,
                             xattrs->ents[num - 1].vsz);
-        error(2,"  [%.*zd] %s <=> %s\n", width, num,
+        error(2,"             [%.*zd] %s <=> %s\n", width, num,
               xattrs->ents[num - 1].key, val);
         free(val);
       }
@@ -492,48 +540,64 @@ void print_single_xattrs(xattrs_type* xattrs)
   }
 }
 
-void print_xattrs_changes(xattrs_type* old,xattrs_type* new) {
+void print_xattrs_changes(xattrs_type* old,xattrs_type* new,
+        DB_ATTR_TYPE force) {
   
   if (compare_xattrs(old,new)==RETFAIL) {
-    error(2,"XAttrs: old = ");
+    error(2," XAttrs   : old = ");
     print_single_xattrs(old);
-    error(2,"        new = ");
+    error(2,"            new = ");
     print_single_xattrs(new);
+  } else if (force) {
+      error(2," XAttrs   : ");
+      print_single_xattrs(new);
   }
   
 }
 
-void print_md_changes(byte*old,byte*new,int len,char* name)
+#ifdef WITH_E2FSATTRS
+char* e2fsattrs2char(unsigned long flags) {
+    char* string = malloc (20 * sizeof (char));
+    int i;
+    for (i = 0 ; i < 19 ; i++) {
+        if (flag_bits[i] & flags) {
+            string[i]=flag_char[i];
+        } else {
+            string[i]='-';
+        }
+    }
+    string[19] = '\0';
+    return string;
+}
+#endif
+
+void print_md_changes(byte*old,byte*new,int len,char* name, DB_ATTR_TYPE force)
 {
-  int ok = 0;
-  if(old!=NULL && new!=NULL){
-    if(bytecmp(old,new,len)!=0){
-      snprintf(oline,part_len,"%s",encode_base64(old,len));
-      snprintf(nline,part_len,"%s",encode_base64(new,len));
-      ok = 1;
+    int mode = 0;
+    if (old!=NULL && new!=NULL) {
+        if(bytecmp(old,new,len)!=0) {
+            snprintf(oline,part_len,"%s",encode_base64(old,len));
+            snprintf(nline,part_len,"%s",encode_base64(new,len));
+            mode = 1;
+        } else if (force) {
+            snprintf(nline,part_len,"%s",encode_base64(new,len));
+            mode = 2;
+        }
+    } else if (old==NULL && new!=NULL) {
+        snprintf(oline,part_len,"<NONE>");
+        snprintf(nline,part_len,"%s",encode_base64(new,len));
+        mode = 1;
+    } else if (old!=NULL && new==NULL) {
+        snprintf(oline,part_len,"%s",encode_base64(old,len));
+        snprintf(nline,part_len,"<NONE>");
+        mode = 1;
     }
-  } else {
-    if(old == NULL && new == NULL){
-      return;
+    if (mode == 1) {
+        error(2,(char*)entry_format,name,oline,nline);
+    } else if (mode == 2) {
+        error(2,(char*)entry_format_justnew,name,' ',nline);
     }
-    if(old==NULL){
-      snprintf(oline,part_len,"<NONE>");
-    } else {
-      snprintf(oline,part_len,"%s",encode_base64(old,len));
-      ok = 1;
-    }
-    /* OLD one */
-    if(new==NULL){
-      snprintf(nline,part_len,"<NONE>");
-    }else {
-      snprintf(nline,part_len,"%s",encode_base64(new,len));
-      ok = 1;
-    }
-  }
-  if(ok)
-    error(2,(char*)entry_format,name,oline,nline);
-  
-  return;
+    return;
 }
 
 int is_time_null(struct tm *ot)
@@ -607,7 +671,7 @@ void print_long_changes(const char* name, AIDE_OFF_TYPE old, AIDE_OFF_TYPE new,
   snprintf(nline,part_len,"%lu",new);
 #endif
   if (justnew) {
-    error(2,(char*)entry_format,name,nline);
+    error(2,(char*)entry_format_justnew,name,' ',nline);
   } else {
     error(2,(char*)entry_format,name,oline,nline);
   }
@@ -620,30 +684,12 @@ void print_string_changes(const char* name, const char* old, const char* new, in
   }
   snprintf(nline,part_len,"%s",new);
   if (justnew) {
-    error(2,(char*)entry_format,name,nline); 
+    error(2,(char*)entry_format_justnew,name,' ',nline);
   } else {
     error(2,(char*)entry_format,name,oline,nline); 
   }
 }
 
-char get_file_type_char(mode_t mode) {
-    if (S_ISREG(mode)) return 'f';
-    else if(S_ISDIR(mode)) return 'd';
-#ifdef S_ISFIFO
-    else if (S_ISFIFO(mode)) return 'F';
-#endif
-    else if (S_ISLNK(mode)) return 'L';
-    else if (S_ISBLK(mode)) return 'B';
-    else if (S_ISCHR(mode)) return 'D';
-#ifdef S_ISSOCK
-    else if (S_ISSOCK(mode)) return 's';
-#endif
-#ifdef S_ISDOOR
-    else if (S_ISDOOR(mode)) return '|';
-#endif
-    else return '?';
-}
-
 char* get_file_type_string(mode_t mode) {
     switch (get_file_type_char(mode)) {
         case 'f': return "File";
@@ -658,20 +704,40 @@ char* get_file_type_string(mode_t mode) {
     }
 }
 
-void print_added_line(db_line* data) {
+void print_simple_line(db_line* data, char* s, char c) {
+    char* tmp=NULL;
+    int i=0;
+    int length = summary_len-1;
+    /* The length depends on compile options */
+#ifdef WITH_ACL
+    length++;
+#endif
+#ifdef WITH_XATTR
+    length++;
+#endif
+#ifdef WITH_SELINUX
+    length++;
+#endif
+#ifdef WITH_E2FSATTRS
+    length++;
+#endif
     if(conf->summarize_changes==1) {
-        error(2,"%c+++++++++++++++: %s\n",get_file_type_char(data->perm) , data->filename);
+        tmp=(char*)malloc(sizeof(char)*(length+1));
+        for(i=0;i<length;i++){ tmp[i]=c; }
+        tmp[length]='\0';
+        error(2,"%c%s: %s\n",get_file_type_char(data->perm) , tmp, data->filename);
+        free(tmp); tmp=NULL;
     } else {
-        error(2,"added: %s\n",data->filename);
+        error(2,"%s: %s\n", s, data->filename);
     }
 }
 
+void print_added_line(db_line* data) {
+    print_simple_line(data, "added", '+');
+}
+
 void print_removed_line(db_line* data) {
-    if(conf->summarize_changes==1) {
-        error(2,"%c---------------: %s\n",get_file_type_char(data->perm), data->filename);
-    } else {
-        error(2,"removed: %s\n",data->filename);
-    }
+    print_simple_line(data, "removed", '-');
 }
 
 int str_has_changed(char*old,char*new)
@@ -689,7 +755,7 @@ int md_has_changed(byte*old,byte*new,int len)
              (old==NULL && new!=NULL)));
 }
 
-char get_size_char(int ignorelist, db_line* old, db_line* new) {
+char get_size_char(DB_ATTR_TYPE ignorelist, db_line* old, db_line* new) {
     if (DB_SIZE&old->attr || DB_SIZEG&old->attr) {
         if ((DB_SIZE&old->attr && DB_SIZE&new->attr) || (DB_SIZEG&old->attr && DB_SIZEG&new->attr)) {
             if (((DB_SIZE&old->attr && DB_SIZE&new->attr) && DB_SIZE&ignorelist) || 
@@ -729,6 +795,8 @@ void print_changed_line(db_line* old,db_line* new, DB_ATTR_TYPE ignorelist) {
         } \
     } else if (a&new->attr) { \
         summary[d]='+'; \
+    } else { \
+        summary[d]=' '; \
     }
 
 #define easy_char(a,b,c,d) \
@@ -746,11 +814,30 @@ void print_changed_line(db_line* old,db_line* new, DB_ATTR_TYPE ignorelist) {
         } \
     } else if (a&new->attr) { \
         summary[d]='+'; \
+    } else { \
+        summary[d]=' '; \
     }
 
     if(conf->summarize_changes==1) {
-        char summary[]="                ";
-        summary[0]=get_file_type_char(new->perm);
+        int offset = 0;
+        int length = summary_len;
+        /* The length depends on compile options */
+#ifdef WITH_ACL
+        length++;
+#endif
+#ifdef WITH_XATTR
+        length++;
+#endif
+#ifdef WITH_SELINUX
+        length++;
+#endif
+#ifdef WITH_E2FSATTRS
+        length++;
+#endif
+        char* summary = malloc ((length+1) * sizeof (char));
+        summary[0]= ((!(DB_FTYPE&ignorelist)) &&
+                (((DB_FTYPE&old->attr && DB_FTYPE&new->attr) &&
+                  get_file_type_char(old->perm)!=get_file_type_char(new->perm)))) ? '!' : get_file_type_char(new->perm);
         easy_compare_char(DB_LINKNAME,str_has_changed(old->linkname,new->linkname),'l',1);
         summary[2]=get_size_char(ignorelist, old, new);
         easy_char(DB_BCOUNT,bcount,'b',3);
@@ -842,14 +929,26 @@ void print_changed_line(db_line* old,db_line* new, DB_ATTR_TYPE ignorelist) {
 #endif        
                 ) {
             summary[12]='.';
+        } else {
+            summary[12]=' ';
         }
-#ifdef WITH_ACL
-        easy_compare_char(DB_ACL,compare_acl(old->acl,new->acl)==RETFAIL,'A',13);
-#endif
-        easy_compare_char(DB_XATTRS,compare_xattrs(old->xattrs,new->xattrs)==RETFAIL,'X',14);
-        easy_compare_char(DB_SELINUX,str_has_changed(old->cntx,new->cntx),'S',15);
 
+#ifdef WITH_ACL
+        easy_compare_char(DB_ACL,compare_acl(old->acl,new->acl)==RETFAIL,'A',summary_len+offset++);
+#endif
+#ifdef WITH_XATTR
+        easy_compare_char(DB_XATTRS,compare_xattrs(old->xattrs,new->xattrs)==RETFAIL,'X',summary_len+offset++);
+#endif
+#ifdef WITH_SELINUX
+        easy_compare_char(DB_SELINUX,str_has_changed(old->cntx,new->cntx),'S',summary_len+offset++);
+#endif
+#ifdef WITH_E2FSATTRS
+        easy_char(DB_E2FSATTRS,e2fsattrs,'E',summary_len+offset++);
+#endif
+        summary[summary_len+offset]='\0';
         error(2,"%s: %s\n",summary, new->filename);
+        free(summary);
+        summary=NULL;
     } else {
         error(2,"changed: %s\n",new->filename);
     }
@@ -867,14 +966,18 @@ void print_dbline_changes(db_line* old,db_line* new,
   forced_attrs&=new->attr;
   
   error(2,"\n%s: %s\n",get_file_type_string(new->perm),new->filename);
-  
-  if(!(DB_LINKNAME&ignorelist)){
-    print_str_changes(old->linkname,new->linkname, "Lname");
+
+  if ((!(DB_FTYPE&ignorelist)) && (((DB_FTYPE&old->attr && DB_FTYPE&new->attr) && get_file_type_char(old->perm)!=get_file_type_char(new->perm)) || DB_FTYPE&forced_attrs)) {
+          print_string_changes("File type", get_file_type_string(old->perm),get_file_type_string(new->perm), get_file_type_char(old->perm)==get_file_type_char(new->perm));
   }
-  if (!(DB_SIZE&ignorelist)) {
-    if(old->size!=new->size||(DB_SIZE&forced_attrs)){
-      print_long_changes("Size", old->size,new->size,old->size==new->size);
-    }
+
+  if(!(DB_LINKNAME&ignorelist)){
+    print_str_changes(old->linkname,new->linkname, "Lname", DB_LINKNAME&forced_attrs);
+  }
+
+  if (((!(DB_SIZEG&ignorelist)) && (((DB_SIZEG&old->attr && DB_SIZEG&new->attr)  && old->size>new->size) || DB_SIZEG&forced_attrs))
+     || ((!(DB_SIZE&ignorelist)) && (((DB_SIZE&old->attr && DB_SIZE&new->attr) && old->size!=new->size) || DB_SIZE&forced_attrs)) ) {
+          print_long_changes("Size", old->size,new->size,old->size==new->size);
   }
 
   if (!(DB_BCOUNT&ignorelist)) {
@@ -883,10 +986,10 @@ void print_dbline_changes(db_line* old,db_line* new,
     }
   }
   if (!(DB_PERM&ignorelist)) {
-    if(old->perm!=new->perm || DB_PERM&forced_attrs){
+    if((DB_PERM&old->attr && DB_PERM&new->attr && old->perm!=new->perm) || DB_PERM&forced_attrs){
       tmp=perm_to_char(old->perm);
       tmp2=perm_to_char(new->perm);
-      print_string_changes("Permissions", tmp,tmp2,old->perm==new->perm);
+      print_string_changes("Perm", tmp,tmp2,old->perm==new->perm);
       free(tmp);
       free(tmp2);
       tmp=NULL;
@@ -925,7 +1028,7 @@ void print_dbline_changes(db_line* old,db_line* new,
   }
 
   if (!(DB_INODE&ignorelist)) {
-    if(old->inode!=new->inode||DB_INODE&forced_attrs){
+    if(((DB_INODE&old->attr && (DB_INODE&new->attr)) && old->inode!=new->inode) ||DB_INODE&forced_attrs){
       print_int_changes("Inode", old->inode,new->inode,old->inode==new->inode);
     }
   }
@@ -938,83 +1041,95 @@ void print_dbline_changes(db_line* old,db_line* new,
   if (!(DB_MD5&ignorelist)) {  
     print_md_changes(old->md5,new->md5,
 		     HASH_MD5_LEN,
-		     "MD5");
+		     "MD5", DB_MD5&forced_attrs);
   }
   
   if (!(DB_SHA1&ignorelist)) {
       print_md_changes(old->sha1,new->sha1,
 		       HASH_SHA1_LEN,
-		       "SHA1");
+		       "SHA1", DB_SHA1&forced_attrs);
   }
 
   if (!(DB_RMD160&ignorelist)) {
     print_md_changes(old->rmd160,new->rmd160,
 		     HASH_RMD160_LEN,
-		     "RMD160");
+		     "RMD160", DB_RMD160&forced_attrs);
   }
   
   if (!(DB_TIGER&ignorelist)) {
     print_md_changes(old->tiger,new->tiger,
 		     HASH_TIGER_LEN,
-		     "TIGER");
+		     "TIGER", DB_TIGER&forced_attrs);
   }
   
   if (!(DB_SHA256&ignorelist)) {
       print_md_changes(old->sha256,new->sha256,
 		       HASH_SHA256_LEN,
-		       "SHA256");
+		       "SHA256", DB_SHA256&forced_attrs);
   }
 
   if (!(DB_SHA512&ignorelist)) {
       print_md_changes(old->sha512,new->sha512,
 		       HASH_SHA512_LEN,
-		       "SHA512");
+		       "SHA512", DB_SHA512&forced_attrs);
   }
 
 #ifdef WITH_MHASH
   if (!(DB_CRC32&ignorelist)) {
     print_md_changes(old->crc32,new->crc32,
 		     HASH_CRC32_LEN,
-		     "CRC32");
+		     "CRC32", DB_CRC32&forced_attrs);
   }
   
   if (!(DB_HAVAL&ignorelist)) {
     print_md_changes(old->haval,new->haval,
 		     HASH_HAVAL256_LEN,
-		     "HAVAL");
+		     "HAVAL", DB_HAVAL&forced_attrs);
   }
   
   if (!(DB_GOST&ignorelist)) {
     print_md_changes(old->gost,new->gost,
 		     HASH_GOST_LEN,
-		     "GOST");
+		     "GOST", DB_GOST&forced_attrs);
   }
   
   if (!(DB_CRC32B&ignorelist)) {
     print_md_changes(old->crc32b,new->crc32b,
 		     HASH_CRC32B_LEN,
-		     "CRC32B");
+		     "CRC32B", DB_CRC32B&forced_attrs);
   }
 
   if (!(DB_WHIRLPOOL&ignorelist)) {
       print_md_changes(old->whirlpool,new->whirlpool,
 		       HASH_WHIRLPOOL_LEN,
-		       "WHIRLPOOL");
+		       "WHIRLPOOL", DB_WHIRLPOOL&forced_attrs);
   }
 #endif                   
 
 #ifdef WITH_ACL
   if (!(DB_ACL&ignorelist)) {
-    print_acl_changes(old->acl,new->acl);
+    print_acl_changes(old->acl,new->acl, DB_ACL&forced_attrs);
   }
 #endif
   if (!(DB_XATTRS&ignorelist)) {
-    print_xattrs_changes(old->xattrs,new->xattrs);
+    print_xattrs_changes(old->xattrs,new->xattrs, DB_XATTRS&forced_attrs);
   }
   if (!(DB_SELINUX&ignorelist)) {
-    print_str_changes(old->cntx,new->cntx, "SELinux");
+    print_str_changes(old->cntx,new->cntx, "SELinux", DB_SELINUX&forced_attrs);
   }
   
+#ifdef WITH_E2FSATTRS
+  if ( !(DB_E2FSATTRS&ignorelist) ) {
+      if(old->e2fsattrs!=new->e2fsattrs || DB_E2FSATTRS&forced_attrs ) {
+          tmp=e2fsattrs2char(old->e2fsattrs);
+          tmp2=e2fsattrs2char(new->e2fsattrs);
+          print_string_changes("E2FSAttrs", tmp, tmp2, old->e2fsattrs==new->e2fsattrs);
+          free(tmp); free(tmp2);
+          tmp=NULL; tmp2=NULL;
+      }
+  }
+#endif
+
   return;
 }
 
@@ -1082,10 +1197,10 @@ void print_report_header(int nfil,int nadd,int nrem,int nchg)
 {
   struct tm* st=localtime(&(conf->start_time));
   if(conf->action&DO_COMPARE)
-    error(0,_("AIDE found differences between database and filesystem!!\n"));
+    error(0,_("AIDE " AIDEVERSION " found differences between database and filesystem!!\n"));
 
   if(conf->action&DO_DIFF)
-    error(0,_("AIDE found differences between the two databases!!\n"));
+    error(0,_("AIDE " AIDEVERSION " found differences between the two databases!!\n"));
   if(conf->config_version)
     error(2,_("Config version used: %s\n"),conf->config_version);
 
@@ -1104,168 +1219,6 @@ void print_report_footer(struct tm* st)
 	  st->tm_hour, st->tm_min, st->tm_sec);
 }
 
-void compare_db(list* new,db_config* dbconf)
-{
-  db_line* old=NULL;
-  list* l=new;
-  list* r=NULL;
-  list* removed=NULL;
-  list* changednew=NULL;
-  list* changedold=NULL;
-  list* added=NULL;
-  long nrem=0;
-  long nchg=0;
-  long nadd=0;
-  long nfil=0;
-  long filesindir=0;
-  DB_ATTR_TYPE tempignore=0;
-  int initdbwarningprinted=0;
-
-  DB_ATTR_TYPE ignorelist;
-  DB_ATTR_TYPE forced_attrs;
-
-  error(200,_("compare_db()\n"));
-
-
-  /* With this we avoid unnecessary checking of removed files. */
-  if(dbconf->action&DO_INIT){
-    initdbwarningprinted=1;
-  } else {
-    /* We have to init the rxlsts since they are copied and then 
-       initialized in gen_list.c */
-    init_rxlst(dbconf->selrxlst);
-    init_rxlst(dbconf->equrxlst);
-    init_rxlst(dbconf->negrxlst);
-  }
-  
-  /* We have a way to ignore some changes... */ 
-  
-  ignorelist=get_ignorelist();
-  
-  forced_attrs=get_report_attributes();
-
-  if (forced_attrs==DB_ATTR_UNDEF) {
-    forced_attrs=0;
-  }
-
-  for(old=db_readline(DB_OLD);old;old=db_readline(DB_OLD)){
-    nfil++;
-    r=find_line_match(old,l);
-    if(r==NULL){
-      /* The WARNING is only printed once */
-      /* FIXME There should be a check for this in changed part also */
-      /* This part should also be rethinked */
-      if(!initdbwarningprinted &&
-	 (check_list_for_match(dbconf->selrxlst,old->filename,&tempignore) ||
-	  check_list_for_match(dbconf->equrxlst,old->filename,&tempignore)) &&
-	 !check_list_for_match(dbconf->negrxlst,old->filename,&tempignore)){
-	if(!(dbconf->action&DO_INIT)){
-	  error(2,_("WARNING: Old db contains a entry that shouldn\'t be there, run --init or --update\n"));
-	}
-	initdbwarningprinted=1;
-      }
-      removed=list_append(removed,(void*)old);
-      nrem++;
-    }else {
-      int localignorelist=old->attr ^ ((db_line*)r->data)->attr;
-      
-      if ((localignorelist&(~(DB_NEWFILE|DB_RMFILE)))!=0) {
-	error(2,"Entry %s in databases has different attributes: %llx %llx\n",
-              old->filename,old->attr,((db_line*)r->data)->attr);
-      }
-      
-      localignorelist|=ignorelist;
-      
-      if(compare_dbline(old,(db_line*)r->data,localignorelist)!=0){
-	changednew=list_append(changednew,r->data);
-	changedold=list_append(changedold,(void*)old);
-	r->data=NULL;
-	l=list_delete_item(r);
-	nchg++;
-      }else {
-	/* This line was ok */
-	/*
-	  Cannot free, check why.
-	  It's because db_disk needs it for going back
-	  to it's parent.
-	*/
-	
-	l=list_delete_item(r);
-      }
-    }
-    
-  }
-  /* Now we have checked the old database and removed the lines *
-   * that have matched. */
-  if(l!=NULL){
-    added=l->header->head;
-  }else {
-    added=NULL;
-  }
-  
-  for(l=added;l;l=l->next){
-    nadd++;
-  }
-
-  if(nadd!=0||nrem!=0||nchg!=0){
-    print_report_header(nfil,nadd,nrem,nchg);
-
-    if(nadd!=0){
-      error(2,_("\n---------------------------------------------------\n"));
-      error(2,_("Added files:\n"));
-      error(2,_("---------------------------------------------------\n\n"));
-      for(r=added;r;r=r->next){
-       print_added_line((db_line*)r->data);
-	if(dbconf->verbose_level<20){
-	  if(S_ISDIR(((db_line*)r->data)->perm)){
-	    eat_files_indir(r->next,((db_line*)r->data)->filename,&filesindir);
-	    if(filesindir>0){
-	      error(2,
-		    _("added: THERE WERE ALSO %li "
-		    "FILES ADDED UNDER THIS DIRECTORY\n")
-		    ,filesindir);
-	    }
-	  }
-	}
-      }
-    }
-    
-
-    if(nrem!=0){
-      error(2,_("\n---------------------------------------------------\n"));
-      error(2,_("Removed files:\n"));
-      error(2,_("---------------------------------------------------\n\n"));
-      for(r=removed;r;r=r->next){
-       print_removed_line((db_line*)r->data);
-      }
-    }
-
-    if(nchg!=0){
-      error(2,_("\n---------------------------------------------------\n"));
-      error(2,_("Changed files:\n"));
-      error(2,_("---------------------------------------------------\n\n"));
-      for(r=changedold,l=changednew;r;r=r->next,l=l->next){
-	print_changed_line((db_line*)r->data,
-			     (db_line*)l->data,ignorelist);
-      }
-    }
-
-    if((dbconf->verbose_level>=5)&&(nchg!=0)){
-      error(2,_("\n---------------------------------------------------\n"));
-      error(2,_("Detailed information about changes:\n"));
-      error(2,_("---------------------------------------------------\n\n"));
-      for(r=changedold,l=changednew;r;r=r->next,l=l->next){
-	DB_ATTR_TYPE localignorelist=((db_line*)l->data)->attr^((db_line*)r->data)->attr;
-	localignorelist|=ignorelist;
-	print_dbline_changes((db_line*)r->data,
-			     (db_line*)l->data,localignorelist,forced_attrs);
-      }
-    }
-    dbconf->end_time=time(&(dbconf->end_time));
-    print_report_footer(localtime(&(dbconf->end_time)));
-  }
-}
-
   /* Something changed, send audit anomaly message */
 void send_audit_report(long nadd, long nrem, long nchg)
 {
@@ -1294,8 +1247,8 @@ void send_audit_report(long nadd, long nrem, long nchg)
 long report_tree(seltree* node,int stage, long* status)
 {
   list* r=NULL;
-  int ignorelist=0;
-  int forced_attrs=0;
+  DB_ATTR_TYPE ignorelist=0;
+  DB_ATTR_TYPE forced_attrs=0;
   int top=0;
 
   ignorelist=get_ignorelist();
@@ -1391,11 +1344,28 @@ long report_tree(seltree* node,int stage, long* status)
       error(2,_("---------------------------------------------------\n\n"));
     }
     if(node->checked&NODE_CHANGED){
-      int localignorelist=(node->old_data->attr ^ node->new_data->attr)|ignorelist;
+      DB_ATTR_TYPE localignorelist=(node->old_data->attr ^ node->new_data->attr)|ignorelist;
       print_dbline_changes(node->old_data,node->new_data,localignorelist,forced_attrs);
     }
   }
 
+  if((stage==5)&&(status[2]||status[3]||status[4])) {
+    if(top){
+      error(2,_("\n---------------------------------------------------\n"));
+      if (status[2]&&status[3]&&status[4]) { error(2,_("Added, removed and changed files:\n")); }
+      else if (status[2]&&status[3]) { error(2,_("Added and removed files:\n")); }
+      else if (status[2]&&status[4]) { error(2,_("Added and changed files:\n")); }
+      else if (status[3]&&status[4]) { error(2,_("Removed and changed files:\n")); }
+      else if (status[2]) { error(2,_("Added files:\n")); }
+      else if (status[3]) { error(2,_("Removed files:\n")); }
+      else if (status[4]) { error(2,_("Changed files:\n")); }
+      error(2,_("---------------------------------------------------\n\n"));
+    }
+    if(node->checked&NODE_ADDED){ print_added_line(node->new_data); }
+    if(node->checked&NODE_REMOVED){ print_removed_line(node->old_data); }
+    if(node->checked&NODE_CHANGED){ print_changed_line(node->old_data,node->new_data,ignorelist); }
+  }
+
   /* All stage dependent things done for this node. Let's check children */
   for(r=node->childs;r;r=r->next){
     report_tree((seltree*)r->data,stage,status);
