@@ -57,53 +57,56 @@
      TSTRING = 273,
      TACLNOSYMLINKFOLLOW = 274,
      TWARNDEADSYMLINKS = 275,
-     TSUMMARIZECHANGES = 276,
-     TNEWLINE = 277,
-     TVERBOSE = 278,
-     TCONFIG_FILE = 279,
-     TDATABASE = 280,
-     TDATABASE_OUT = 281,
-     TDATABASE_NEW = 282,
-     TREPORT_URL = 283,
-     TGZIPDBOUT = 284,
-     TUMASK = 285,
-     TTRUE = 286,
-     TFALSE = 287,
-     TRECSTOP = 288,
-     TCONFIG_VERSION = 289,
-     TSELRXRULE = 290,
-     TEQURXRULE = 291,
-     TNEGRXRULE = 292,
-     TRIGHTS = 293,
-     TUSER = 294,
-     TGROUP = 295,
-     TINODE = 296,
-     TLINKCOUNT = 297,
-     TSIZE = 298,
-     TGROWINGSIZE = 299,
-     TATIME = 300,
-     TCTIME = 301,
-     TMTIME = 302,
-     TACL = 303,
-     TXATTRS = 304,
-     TSELINUX = 305,
-     TTIGER = 306,
-     TSHA1 = 307,
-     TRMD160 = 308,
-     TMD2 = 309,
-     TMD4 = 310,
-     TMD5 = 311,
-     TSHA256 = 312,
-     TSHA512 = 313,
-     TWHIRLPOOL = 314,
-     TL = 315,
-     TR = 316,
-     TGZIPHEADER = 317,
-     TDBSPEC = 318,
-     TUNKNOWN = 319,
-     TNAME = 320,
-     TERROR = 321,
-     TEOF = 322
+     TGROUPED = 276,
+     TSUMMARIZECHANGES = 277,
+     TNEWLINE = 278,
+     TVERBOSE = 279,
+     TCONFIG_FILE = 280,
+     TDATABASE = 281,
+     TDATABASE_OUT = 282,
+     TDATABASE_NEW = 283,
+     TREPORT_URL = 284,
+     TGZIPDBOUT = 285,
+     TUMASK = 286,
+     TTRUE = 287,
+     TFALSE = 288,
+     TRECSTOP = 289,
+     TCONFIG_VERSION = 290,
+     TSELRXRULE = 291,
+     TEQURXRULE = 292,
+     TNEGRXRULE = 293,
+     TRIGHTS = 294,
+     TUSER = 295,
+     TGROUP = 296,
+     TINODE = 297,
+     TLINKCOUNT = 298,
+     TFTYPE = 299,
+     TSIZE = 300,
+     TGROWINGSIZE = 301,
+     TATIME = 302,
+     TCTIME = 303,
+     TMTIME = 304,
+     TACL = 305,
+     TXATTRS = 306,
+     TSELINUX = 307,
+     TE2FSATTRS = 308,
+     TTIGER = 309,
+     TSHA1 = 310,
+     TRMD160 = 311,
+     TMD2 = 312,
+     TMD4 = 313,
+     TMD5 = 314,
+     TSHA256 = 315,
+     TSHA512 = 316,
+     TWHIRLPOOL = 317,
+     TL = 318,
+     TR = 319,
+     TGZIPHEADER = 320,
+     TDBSPEC = 321,
+     TUNKNOWN = 322,
+     TNAME = 323,
+     TERROR = 324,
+     TEOF = 325
    };
 #endif
 /* Tokens.  */
@@ -125,66 +128,69 @@
 #define TSTRING 273
 #define TACLNOSYMLINKFOLLOW 274
 #define TWARNDEADSYMLINKS 275
-#define TSUMMARIZECHANGES 276
-#define TNEWLINE 277
-#define TVERBOSE 278
-#define TCONFIG_FILE 279
-#define TDATABASE 280
-#define TDATABASE_OUT 281
-#define TDATABASE_NEW 282
-#define TREPORT_URL 283
-#define TGZIPDBOUT 284
-#define TUMASK 285
-#define TTRUE 286
-#define TFALSE 287
-#define TRECSTOP 288
-#define TCONFIG_VERSION 289
-#define TSELRXRULE 290
-#define TEQURXRULE 291
-#define TNEGRXRULE 292
-#define TRIGHTS 293
-#define TUSER 294
-#define TGROUP 295
-#define TINODE 296
-#define TLINKCOUNT 297
-#define TSIZE 298
-#define TGROWINGSIZE 299
-#define TATIME 300
-#define TCTIME 301
-#define TMTIME 302
-#define TACL 303
-#define TXATTRS 304
-#define TSELINUX 305
-#define TTIGER 306
-#define TSHA1 307
-#define TRMD160 308
-#define TMD2 309
-#define TMD4 310
-#define TMD5 311
-#define TSHA256 312
-#define TSHA512 313
-#define TWHIRLPOOL 314
-#define TL 315
-#define TR 316
-#define TGZIPHEADER 317
-#define TDBSPEC 318
-#define TUNKNOWN 319
-#define TNAME 320
-#define TERROR 321
-#define TEOF 322
+#define TGROUPED 276
+#define TSUMMARIZECHANGES 277
+#define TNEWLINE 278
+#define TVERBOSE 279
+#define TCONFIG_FILE 280
+#define TDATABASE 281
+#define TDATABASE_OUT 282
+#define TDATABASE_NEW 283
+#define TREPORT_URL 284
+#define TGZIPDBOUT 285
+#define TUMASK 286
+#define TTRUE 287
+#define TFALSE 288
+#define TRECSTOP 289
+#define TCONFIG_VERSION 290
+#define TSELRXRULE 291
+#define TEQURXRULE 292
+#define TNEGRXRULE 293
+#define TRIGHTS 294
+#define TUSER 295
+#define TGROUP 296
+#define TINODE 297
+#define TLINKCOUNT 298
+#define TFTYPE 299
+#define TSIZE 300
+#define TGROWINGSIZE 301
+#define TATIME 302
+#define TCTIME 303
+#define TMTIME 304
+#define TACL 305
+#define TXATTRS 306
+#define TSELINUX 307
+#define TE2FSATTRS 308
+#define TTIGER 309
+#define TSHA1 310
+#define TRMD160 311
+#define TMD2 312
+#define TMD4 313
+#define TMD5 314
+#define TSHA256 315
+#define TSHA512 316
+#define TWHIRLPOOL 317
+#define TL 318
+#define TR 319
+#define TGZIPHEADER 320
+#define TDBSPEC 321
+#define TUNKNOWN 322
+#define TNAME 323
+#define TERROR 324
+#define TEOF 325
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 44 "conf_yacc.y"
+#line 45 "conf_yacc.y"
 {
   char* s;
   DB_ATTR_TYPE i;
 }
 /* Line 1489 of yacc.c.  */
-#line 188 "conf_yacc.h"
+#line 194 "conf_yacc.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
