@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2006,2010 Rami Lehti, Pablo Virolainen, Richard
+ * van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -84,6 +85,7 @@ const char* db_names[db_unknown+1] = {
    "whirlpool",
    "selinux",
    "xattrs",
+   "e2fsattrs",
    "unknown"} ;
 
 const int db_value[db_unknown+1] = {
@@ -120,6 +122,7 @@ const int db_value[db_unknown+1] = {
    db_whirlpool,        /* "whirlpool", */
    db_selinux,          /* "selinux", */
    db_xattrs,           /* "xattrs",  */
+   db_e2fsattrs,        /* "e2fsattrs",  */
    db_unknown };        /* "unknown"  */
 
 const char* db_namealias[db_alias_size] = {
@@ -370,6 +373,7 @@ db_line* db_char2line(char** ss,int db){
   line->linkname=NULL;
   line->acl=NULL;
   line->xattrs=NULL;
+  line->e2fsattrs=0;
   line->cntx=NULL;
   
   line->attr=conf->attr; /* attributes from @@dbspec */
@@ -591,6 +595,11 @@ db_line* db_char2line(char** ss,int db){
       break;
     }
     
+    case db_e2fsattrs : {
+      line->e2fsattrs=readlong(ss[(*db_order)[i]],"e2fsattrs");
+      break;
+    }
+
     case db_unknown : {
       /* Unknown fields are ignored. */
       break;
