@@ -1,7 +1,8 @@
 /* aide, Advanced Intrusion Detection Environment
  * vi: ts=2 sw=2
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2006,2010 Rami Lehti, Pablo Virolainen, Richard
+ * van den Berg, Mike Markley, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -164,7 +165,7 @@ void add_child (db_line * fil)
 		new_r->checked |= NODE_CHECKED;
 		new_r->checked |= NODE_TRAVERSE;
 	}
-	r->childs = list_append (r->childs, new_r);
+	r->childs = list_sorted_insert (r->childs, new_r, compare_node_by_path);
 }
 
 /*
