@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2007,2010 Rami Lehti, Pablo Virolainen, Mike
+ * Markley, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -1066,6 +1067,12 @@ int db_writeline_file(db_line* line,db_config* dbconf, url_t* url){
 	db_write_byte_base64((byte*)line->cntx, 0, dbconf->db_out, i, 1, 1);
       break;
     }
+#ifdef WITH_E2FSATTRS
+    case db_e2fsattrs : {
+      db_writelong(line->e2fsattrs,dbconf->db_out,i);
+      break;
+    }
+#endif
     case db_checkmask : {
       db_writeoct(line->attr,dbconf->db_out,i);
       break;
