@@ -1,7 +1,8 @@
 /* aide, Advanced Intrusion Detection Environment
  * vi: ts=8 sw=8
  *
- * Copyright (C) 1999,2000,2001,2002,2005 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2002,2004-2006,2009,2010 Rami Lehti, Pablo Virolainen,
+ * Mike Markley, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -397,11 +398,7 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
 
 void fs2db_line(struct AIDE_STAT_TYPE* fs,db_line* line) {
   
-  if(DB_INODE&line->attr||DB_CHECKINODE&line->attr){
-    line->inode=fs->st_ino;
-  } else {
-    line->inode=0;
-  }
+  line->inode=fs->st_ino;
 
   if(DB_UID&line->attr) {
     line->uid=fs->st_uid;
@@ -538,6 +535,22 @@ void acl2line(db_line* line) {
 #endif
 }
 
+#ifdef WITH_E2FSATTRS
+void e2fsattrs2line(db_line* line) {
+    unsigned long flags;
+    if (DB_E2FSATTRS&line->attr) {
+        if (fgetflags(line->filename, &flags) == 0) {
+            line->e2fsattrs=flags;
+        } else {
+            line->attr&=(~DB_E2FSATTRS);
+            line->e2fsattrs=0;
+        }
+    } else {
+        line->e2fsattrs=0;
+    }
+}
+#endif
+
 void no_hash(db_line* line) {
   line->attr&=~DB_HASHES;
 }
