@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
+ * Copyright (C) 1999-2006,2009,2010 Rami Lehti,Pablo Virolainen, Mike
+ * Markley, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -58,6 +59,13 @@ void fs2db_line(struct AIDE_STAT_TYPE* fs,db_line* line);
 void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line);
 void no_hash(db_line* line);
 
+int compare_node_by_path(const void *n1, const void *n2)
+{
+    const seltree *x1 = n1;
+    const seltree *x2 = n2;
+    return strcmp(x1->path, x2->path);
+}
+
 char* strrxtok(char* rx)
 {
   char*p=NULL;
@@ -275,7 +283,7 @@ seltree* new_seltree_node(
       }
     }
     free(tmprxtok);
-    parent->childs=list_append(parent->childs,(void*)node);
+    parent->childs=list_sorted_insert(parent->childs,(void*)node, compare_node_by_path);
     node->parent=parent;
   }else {
     node->parent=NULL;
@@ -613,11 +621,7 @@ list* add_file_to_list(list* listp,char*filename,DB_ATTR_TYPE attr,int* addok)
   }
 
   
-  if(DB_INODE&fil->attr||DB_CHECKINODE&fil->attr){
-    fil->inode=fs.st_ino;
-  } else {
-    fil->inode=0;
-  }
+  fil->inode=fs.st_ino;
 
   if(DB_UID&fil->attr) {
     fil->uid=fs.st_uid;
@@ -1010,7 +1014,8 @@ void strip_dbline(db_line* line,DB_ATTR_TYPE attr)
   if(!(attr&DB_LINKNAME)){
     checked_free(line->linkname);
   }
-  /* permissions are always needed for summarize_changes, hence it is never stripped */
+  /* permissions are always needed for file type detection, hence they are
+   * never stripped */
   if(!(attr&DB_UID)){
     line->uid=0;
   }
@@ -1026,9 +1031,8 @@ void strip_dbline(db_line* line,DB_ATTR_TYPE attr)
   if(!(attr&DB_MTIME)){
     line->mtime=0;
   }
-  if(!(attr&DB_INODE)){
-    line->inode=0;
-  }
+  /* inode is always needed for ignoring changed filename, hence it is
+   * never stripped */
   if(!(attr&DB_LNKCOUNT)){
     line->nlink=0;
   }
@@ -1096,6 +1100,7 @@ void strip_dbline(db_line* line,DB_ATTR_TYPE attr)
     checked_free(line->cntx);
   }
 #endif
+  /* e2fsattrs is stripped within e2fsattrs2line in do_md */
 }
 
 /*
@@ -1200,19 +1205,22 @@ void add_file_to_tree(seltree* tree,db_line* file,int db,int status,
         oldData = node->old_data;
       }
 
-      localignorelist=(oldData->attr^newData->attr)&(~(DB_NEWFILE|DB_RMFILE));
+      localignorelist=(oldData->attr^newData->attr)&(~(DB_NEWFILE|DB_RMFILE|DB_CHECKINODE));
 
       if (localignorelist!=0) {
-        error(5,"Entry \"%s\" \"%s\" in databases has different attributes (here3): %llx %llx\n",
-  	    newData->filename,oldData->filename,oldData->attr,newData->attr);
-      }
-    
-      localignorelist|=ignorelist|DB_CTIME;
-
-      /* Free the data if same else leave as is for report_tree */
-      if(compare_dbline(oldData, newData, localignorelist)==RETOK){
-        node->checked |= db==DB_NEW ? NODE_MOVED_IN : NODE_MOVED_OUT;
-        moved_node->checked |= db==DB_NEW ? NODE_MOVED_OUT : NODE_MOVED_IN;
+         error(220,"Ignoring moved entry (\"%s\" [%llx] => \"%s\" [%llx]) due to different attributes: %llx\n",
+                 oldData->filename, oldData->attr, newData->filename, newData->attr, localignorelist);
+     } else {
+         /* Free the data if same else leave as is for report_tree */
+         if(compare_dbline(oldData, newData, ignorelist|DB_CTIME)==RETOK){
+             node->checked |= db==DB_NEW ? NODE_MOVED_IN : NODE_MOVED_OUT;
+             moved_node->checked |= db==DB_NEW ? NODE_MOVED_OUT : NODE_MOVED_IN;
+             error(220,_("Entry was moved: %s [%llx] => %s [%llx]\n"),
+                     oldData->filename , oldData->attr, newData->filename, newData->attr);
+         } else {
+             error(220,"Ignoring moved entry (\"%s\" => \"%s\") because the entries mismatch\n",
+                     oldData->filename, newData->filename);
+         }
       }
     }
   }
@@ -1349,6 +1357,10 @@ db_line* get_file_attrs(char* filename,DB_ATTR_TYPE attr)
 
   selinux2line(line);
 
+#ifdef WITH_E2FSATTRS
+    e2fsattrs2line(line);
+#endif
+
   if (attr&DB_HASHES && S_ISREG(fs.st_mode)) {
     calc_md(&fs,line);
   } else {
